@@ -22,22 +22,37 @@
 
 #include <QtCore/QObject>
 #include <QtCore/QString>
+#include <QtCore/QStringList>
+
+class BugzillaManager;
 
 class ProductMapping: public QObject
 {
 Q_OBJECT
 public:
-    explicit ProductMapping(const QString&, QObject * parent = 0);
+    explicit ProductMapping(const QString&, BugzillaManager *, QObject * parent = 0);
     
     QString bugzillaProduct() const;
     QString bugzillaComponent() const;
-    
+    QString bugzillaVersion() const;
+    QStringList relatedBugzillaProducts() const;
+
+private Q_SLOTS:
+    void delayedLoginCheck(bool);
+    void checkValidVersions(const QStringList &);
+
 private:
     void map(const QString&);
     void mapUsingInternalFile(const QString&);
-    
+    void getRelatedProductsUsingInternalFile(const QString&);
+
+    QStringList m_relatedBugzillaProducts;
     QString     m_bugzillaProduct;
     QString     m_bugzillaComponent;
+
+    QString     m_bugzillaVersionString;
+
+    BugzillaManager *   m_bugzillaManagerPtr;
 };
 
 #endif
