@@ -32,13 +32,21 @@
 #include <QtDBus/QDBusMetaType>
 #include <QtDBus/QDBusObjectPath>
 
+#include "klocale.h"
 #include "kglobalaccel.h"
 #include "kglobalsettings.h"
+#include "knotification.h"
 #include "kdebug.h"
 
 
 struct KGlobalAccelDPrivate
     {
+
+    KGlobalAccelDPrivate(KGlobalAccelD *q)
+       : q(q)
+       {}
+
+
     GlobalShortcut *findAction(const QStringList &actionId) const;
 
     /**
@@ -52,19 +60,40 @@ struct KGlobalAccelDPrivate
 
     GlobalShortcut *addAction(const QStringList &actionId);
     KdeDGlobalAccel::Component *component(const QStringList &actionId) const;
-    QTimer writeoutTimer;
+
+
+    //! Use KNotification to inform the user new global shortcuts were
+    //! registered
+    void _k_newGlobalShortcutNotification();
 
     void splitComponent(QString &component, QString &context) const
         {
         context = "default";
         if (component.indexOf('|')!=-1)
             {
-            QStringList tmp = component.split("|");
+            QStringList tmp = component.split('|');
             Q_ASSERT(tmp.size()==2);
             component= tmp.at(0);
             context= tmp.at(1);
             }
         }
+
+    enum ChangeType
+        {
+        NewShortcut
+        };
+
+    // List if changed components for _k_globalShortcutNotification
+    QMap<QString, ChangeType> changedComponents;
+
+    //! Timer for delayed popup on new shortcut registration
+    QTimer popupTimer;
+
+    //! Timer for delayed writing to kglobalshortcutsrc
+    QTimer writeoutTimer;
+
+    //! Our holder
+    KGlobalAccelD *q;
     };
 
 
@@ -151,10 +180,11 @@ GlobalShortcut *KGlobalAccelDPrivate::addAction(const QStringList &actionId)
     Q_ASSERT(actionId.size() >= 4);
 
     QString componentUnique = actionId.at(KGlobalAccel::ComponentUnique);
+
     QString contextUnique = "default";
 
     if (componentUnique.indexOf("|")!=-1) {
-        QStringList tmp = componentUnique.split("|");
+        QStringList tmp = componentUnique.split('|');
         Q_ASSERT(tmp.size()==2);
         componentUnique = tmp.at(0);
         contextUnique = tmp.at(1);
@@ -173,6 +203,9 @@ GlobalShortcut *KGlobalAccelDPrivate::addAction(const QStringList &actionId)
     }
 
     Q_ASSERT(!component->getShortcutByName(componentUnique, contextUnique));
+    changedComponents.insert(actionId.at(KGlobalAccel::ComponentUnique), NewShortcut);
+
+    if (!popupTimer.isActive()) popupTimer.start(500);
 
     return new GlobalShortcut(
             actionId.at(KGlobalAccel::ActionUnique),
@@ -180,11 +213,52 @@ GlobalShortcut *KGlobalAccelDPrivate::addAction(const QStringList &actionId)
             component->shortcutContext(contextUnique));
 }
 
+
+void KGlobalAccelDPrivate::_k_newGlobalShortcutNotification()
+{
+    Q_FOREACH(const QString &uniqueName, changedComponents.keys())
+        {
+        kDebug() << "Showing Notification for component" << uniqueName;
+
+        KdeDGlobalAccel::Component *component = GlobalShortcutsRegistry::self()->getComponent(uniqueName);
+        if (!component)
+            {
+            // Can happen if a component is removed immediately after
+            // registering it. kdelibs/kdeui/tests/kglobalshortcuttests does
+            // it for example.
+            continue;
+            }
+
+        KNotification *notification = new KNotification(
+                "newshortcutregistered",
+                KNotification::CloseOnTimeout,
+                q->parent());
+
+        notification->setText(
+                i18n("The application %1 has registered a new global shortcut",
+                        component->friendlyName()));
+
+        notification->setActions( QStringList( i18n( "Open Global Shortcuts Editor" ) ) );
+
+        notification->addContext( "application", component->friendlyName() );
+
+        QObject::connect(notification, SIGNAL(action1Activated()),
+                component, SLOT(showKCM()));
+
+        notification->sendEvent();
+        }
+
+    changedComponents.clear();
+}
+
+
+
+
 Q_DECLARE_METATYPE(QStringList)
 
 KGlobalAccelD::KGlobalAccelD(QObject* parent)
 : QObject(parent),
-   d(new KGlobalAccelDPrivate)
+   d(new KGlobalAccelDPrivate(this))
 {}
 
 
@@ -204,6 +278,10 @@ bool KGlobalAccelD::init()
     connect(&d->writeoutTimer, SIGNAL(timeout()),
             reg, SLOT(writeSettings()));
 
+    d->popupTimer.setSingleShot(true);
+    connect(&d->popupTimer, SIGNAL(timeout()),
+            this, SLOT(_k_newGlobalShortcutNotification()));
+
     if (!QDBusConnection::sessionBus().registerService(QLatin1String("org.kde.kglobalaccel"))) {
         kWarning() << "Failed to register service org.kde.kglobalaccel";
         return false;
@@ -416,7 +494,7 @@ QList<KGlobalShortcutInfo> KGlobalAccelD::getGlobalShortcutsByKey(int key) const
         GlobalShortcutsRegistry::self()->getShortcutsByKey(key);
 
     QList<KGlobalShortcutInfo> rc;
-    Q_FOREACH(GlobalShortcut const*sc, shortcuts)
+    Q_FOREACH(const GlobalShortcut *sc, shortcuts)
         {
 #ifdef KDEDGLOBALACCEL_TRACE
     kDebug() << sc->context()->uniqueName() << sc->uniqueName();
